#modules/copyquestion.py

import tkinter as tk
from tkinter import messagebox
import customtkinter as CTk
import json
from pathlib import Path
import sys  
from .copymanager import CopyManager

class CopyQuestionPopup(CTk.CTkToplevel):
    def __init__(self, parent, default_font="Arial"):
        super().__init__(parent)
        self.title("동형문제 복사기")
        
        self.geometry("1250x680")
        self.minsize(width=1250, height=680)
        self.grab_set()
        self.focus_set()

        # Load the last used font size
        self.default_font_size = self.load_font_size()
        self.default_font = (default_font, self.default_font_size)  # Adjust as needed

        # Ensure the popup window itself is resizable
        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

        # Create the main content frame
        content_frame = CTk.CTkFrame(self, fg_color="transparent", border_width=0, corner_radius=0)
        content_frame.grid(row=0, column=0, sticky="nsew", padx=7, pady=0)
        content_frame.grid_rowconfigure(1, weight=1)
        content_frame.grid_columnconfigure(0, weight=1)

        # Create the label frame
        label_frame = CTk.CTkFrame(content_frame, fg_color="transparent", border_width=0, corner_radius=0)
        label_frame.grid(row=0, column=0, sticky="ew", pady=(0, 5))

        # Create the text frame
        text_frame = CTk.CTkFrame(content_frame, fg_color="transparent", border_width=0, corner_radius=0)
        text_frame.grid(row=1, column=0, sticky="nsew")
        text_frame.grid_rowconfigure(0, weight=1)

        # Configure column ratios for label_frame and text_frame
        for frame in (label_frame, text_frame):
            frame.grid_columnconfigure(0, weight=10)  # 지문
            frame.grid_columnconfigure(1, weight=10)  # 문제
            frame.grid_columnconfigure(2, weight=1)   # 정답
            frame.grid_columnconfigure(3, weight=10)  # 해설

        # Calculate base width for textboxes
        base_width = 100  # Adjust this value as needed

        # Create labels
        labels = ["지문", "문제", "정답", "해설"]
        ratios = [10, 10, 1, 10]

        for i, (label_text, ratio) in enumerate(zip(labels, ratios)):
            label = CTk.CTkLabel(
                label_frame, 
                text=label_text, 
                font=(self.default_font[0], 13, 'bold'), 
                height=20, 
                fg_color="transparent",
                width=base_width * ratio  # Set width based on ratio
            )
            label.grid(row=0, column=i, sticky="ew", padx=3)

        # Initialize self.text_edits
        self.text_edits = []

        # Create text_edits for 지문, 문제, 정답, 해설 (self.text_edits[0] to self.text_edits[3])
        for i, width_ratio in enumerate([10, 10, 1, 10]):
            text_edit = CTk.CTkTextbox(
                master=text_frame,
                border_width=1,
                corner_radius=0,
                text_color=("black", "#DBE1EC"),
                fg_color=("white", "#353638"),
                font=(self.default_font[0], self.default_font_size),
                activate_scrollbars=True,
                undo=True,
                wrap="word",
                width=base_width * width_ratio  # Set minimum width based on ratio
            )
            text_edit.grid(row=0, column=i, sticky="nsew", padx=3, pady=(0, 10))
            # Removed direct insertion of values
            # text_edit.insert("0.0", values[i])  # Insert corresponding value
            self.text_edits.append(text_edit)

        # $출제창안내문(sheet_name)
        self.출제창안내문 = CTk.CTkLabel(
            self, 
            text="이곳에 문제의 샘플을 입력해두면 이와 동일한 유형으로 문제를 제작합니다. '지문', '문제', '정답', '해설'을 각각 적어주세요. ('해설'은 비워두어도 됩니다)\n[주의사항] 지문에 밑줄이 필요한 경우 밑줄 대신 대괄호 [ ]를 사용하고 문제에서도 '밑줄' 대신 '괄호'란 말을 사용하세요.", 
            font=(self.default_font[0], 12), 
            height=40, 
            text_color="black", 
            fg_color="white"
        )
        self.출제창안내문.grid(row=1, column=0, pady=1, padx=10, sticky="ew")

        # Create a frame for buttons and preset controls, placed at the bottom
        button_preset_frame = CTk.CTkFrame(
            self, 
            fg_color="#2B3D2D", 
            border_width=0, 
            corner_radius=3, 
            height=70
        )
        button_preset_frame.grid(row=2, column=0, sticky="ew", padx=10, pady=10)
        button_preset_frame.grid_propagate(False)  # Prevent frame from shrinking to fit its contents

        # Preset Controls Frame (Left Side)
        preset_controls_frame = CTk.CTkFrame(button_preset_frame, fg_color="transparent")
        preset_controls_frame.pack(side=tk.LEFT, padx=(0, 20), pady=0)

        # Preset Combobox Label - Changed to "불러오기:" as per user request
        CTk.CTkLabel(preset_controls_frame, text="불러오기:", font=(self.default_font[0], 12), text_color="white").pack(side=tk.LEFT, padx=(20, 5))

        # Preset Combobox
        self.preset_combobox = CTk.CTkComboBox(
            preset_controls_frame, 
            font=(self.default_font[0], 12), 
            command=self.on_preset_selected, 
            state='readonly', 
            width=150
        )
        self.preset_combobox.pack(side=tk.LEFT, padx=5)
        self.preset_combobox.set("불러올 프리셋을 선택하세요")
        # Preset Name Entry
        self.preset_name_entry = CTk.CTkEntry(
            preset_controls_frame, 
            width=150, 
            placeholder_text="저장할 프리셋 이름"
        )
        self.preset_name_entry.pack(side=tk.LEFT, padx=5)

        # Save Preset Button - Consistent styling as per user request
        save_preset_button = CTk.CTkButton(
            preset_controls_frame, 
            text="저장", 
            command=self.save_preset, 
            width=60, 
            height=25,
            fg_color="white", 
            hover_color="#DDA15C", 
            text_color='black', 
            font=(self.default_font[0], 12, 'bold')
        )
        save_preset_button.pack(side=tk.LEFT, padx=5)

        # Delete Preset Button - Consistent styling as per user request
        delete_preset_button = CTk.CTkButton(
            preset_controls_frame, 
            text="삭제", 
            command=self.delete_preset, 
            width=60, 
            height=25,
            fg_color="white", 
            hover_color="#DDA15C", 
            text_color='black', 
            font=(self.default_font[0], 12, 'bold')
        )
        delete_preset_button.pack(side=tk.LEFT, padx=5)

        # "저장 후 닫기" Button (Right Side)
        save_close_button = CTk.CTkButton(
            button_preset_frame, 
            text="저장 후 닫기", 
            fg_color="#FEF9E0", 
            hover_color="#DDA15C", 
            text_color='black', 
            font=(self.default_font[0], 12, 'bold'), 
            command=self.save_and_close, 
            width=110, 
            height=30
        )
        save_close_button.pack(side=tk.RIGHT, padx=10)

        # Define the application data directory
        self.app_data_dir = Path.home() / ".my_application_data"
        self.app_data_dir.mkdir(exist_ok=True)  # Create the directory if it does not exist

        # Preset File Path within app_data_dir
        self.presets_file = self.app_data_dir / "copyquestion_presets.json"

        # Last Copy Question Content File Path
        self.last_copy_question_file = self.app_data_dir / "last_copy_question.txt"

        # Load presets and last copy question content
        self.load_presets()
        self.load_last_copy_question()



        if sys.platform.startswith('darwin'):            
            # Initialize the CopyManager with the current window
            self.copy_manager = CopyManager(self)
            
            self.text_edits[0].bind("<KeyPress>", self.copy_manager.handle_keypress)
            self.text_edits[1].bind("<KeyPress>", self.copy_manager.handle_keypress)
            self.text_edits[2].bind("<KeyPress>", self.copy_manager.handle_keypress)
            self.text_edits[3].bind("<KeyPress>", self.copy_manager.handle_keypress)

        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()


    def load_font_size(self):
        """
        Load the last used font size.
        For demonstration, return a fixed size.
        """
        # Implement actual loading logic if you have saved font sizes
        return 12  # Default font size

    def load_presets(self):
        """
        Load presets from the JSON file located in app_data_dir.
        """
        if self.presets_file.exists():
            try:
                with open(self.presets_file, 'r', encoding='utf-8') as file:
                    self.presets = json.load(file)
            except json.JSONDecodeError:
                messagebox.showerror("Error", "프리셋 파일을 불러오는 중 오류가 발생했습니다.")
                self.presets = {}
        else:
            self.presets = {}
        self.preset_combobox.configure(values=list(self.presets.keys()))
        if self.presets:
            self.preset_combobox.set("불러올 프리셋을 선택하세요")
            #self.preset_combobox.set(list(self.presets.keys())[0])
        else:
            self.preset_combobox.set("")

    def load_last_copy_question(self):
        """
        Load the last saved copy_question_content into the text_edit widgets.
        """
        if self.last_copy_question_file.exists():
            try:
                with open(self.last_copy_question_file, 'r', encoding='utf-8') as file:
                    content = file.read()
                
                # Split the content into sections based on the predefined format
                sections = content.split("\n")
                # Initialize variables to store each section
                passage = ""
                question = ""
                answer = ""
                explanation = ""
                
                current_section = None
                for line in sections:
                    if line.startswith("[Passage]"):
                        current_section = "passage"
                        passage += line.replace("[Passage]", "").strip() + "\n"
                    elif line.startswith("[Question]"):
                        current_section = "question"
                        question += line.replace("[Question]", "").strip() + "\n"
                    elif line.startswith("[Answer]"):
                        current_section = "answer"
                        answer += line.replace("[Answer]", "").strip() + "\n"
                    elif line.startswith("[Explanation]"):
                        current_section = "explanation"
                        explanation += line.replace("[Explanation]", "").strip() + "\n"
                    else:
                        if current_section == "passage":
                            passage += line + "\n"
                        elif current_section == "question":
                            question += line + "\n"
                        elif current_section == "answer":
                            answer += line + "\n"
                        elif current_section == "explanation":
                            explanation += line + "\n"
                
                # Remove the last newline character
                passage = passage.strip()
                question = question.strip()
                answer = answer.strip()
                explanation = explanation.strip()
                if explanation == "":
                    explanation = "(Write your own explanation on the answer(s))"

                # Insert the loaded content into the text_edit widgets
                loaded_values = [passage, question, answer, explanation]
                for i, value in enumerate(loaded_values):
                    self.text_edits[i].delete("1.0", "end")
                    self.text_edits[i].insert("1.0", value)
                
            except Exception as e:
                messagebox.showerror("Error", f"Failed to load last copy question content: {e}")
        else:
            # If the file doesn't exist, initialize with empty strings or default values
            default_values = ["", "", "", ""]
            for i, value in enumerate(default_values):
                self.text_edits[i].delete("1.0", "end")
                self.text_edits[i].insert("1.0", value)

    def save_preset(self):
        """
        Save the current text fields as a new preset.
        """
        preset_name = self.preset_name_entry.get().strip()
        if not preset_name:
            messagebox.showwarning("Warning", "프리셋 이름을 입력해주세요.")
            return
        if preset_name in self.presets:
            overwrite = messagebox.askyesno("Overwrite", f"프리셋 '{preset_name}'을(를) 덮어쓰시겠습니까?")
            if not overwrite:
                return
        # Collect current data
        passage = self.text_edits[0].get("1.0", "end-1c").strip()    # 지문
        question = self.text_edits[1].get("1.0", "end-1c").strip()   # 문제
        answer = self.text_edits[2].get("1.0", "end-1c").strip()     # 정답
        explanation = self.text_edits[3].get("1.0", "end-1c").strip() # 해설

        # Save to presets
        self.presets[preset_name] = {
            "지문": passage,
            "문제": question,
            "정답": answer,
            "해설": explanation
        }
        self.save_presets_to_file()
        self.preset_combobox.configure(values=list(self.presets.keys()))
        self.preset_combobox.set(preset_name)
        self.preset_name_entry.delete(0, tk.END)
        messagebox.showinfo("Success", f"프리셋 '{preset_name}'이(가) 저장되었습니다.")

    def delete_preset(self):
        """
        Delete the selected preset.
        """
        preset_name = self.preset_combobox.get()
        if not preset_name:
            messagebox.showwarning("Warning", "삭제할 프리셋을 선택해주세요.")
            return
        confirm = messagebox.askyesno("Confirm Delete", f"프리셋 '{preset_name}'을(를) 삭제하시겠습니까?")
        if confirm:
            del self.presets[preset_name]
            self.save_presets_to_file()
            self.preset_combobox.configure(values=list(self.presets.keys()))
            if self.presets:
                self.preset_combobox.set(list(self.presets.keys())[0])
            else:
                self.preset_combobox.set("")
            messagebox.showinfo("Deleted", f"프리셋 '{preset_name}'이(가) 삭제되었습니다.")

    def on_preset_selected(self, preset_name):
        """
        Load the selected preset into the text fields.
        """
        if preset_name in self.presets:
            preset = self.presets[preset_name]
            self.text_edits[0].delete("1.0", "end")
            self.text_edits[0].insert("1.0", preset["지문"])
            self.text_edits[1].delete("1.0", "end")
            self.text_edits[1].insert("1.0", preset["문제"])
            self.text_edits[2].delete("1.0", "end")
            self.text_edits[2].insert("1.0", preset["정답"])
            self.text_edits[3].delete("1.0", "end")
            self.text_edits[3].insert("1.0", preset["해설"])

    def save_presets_to_file(self):
        """
        Save the current presets to the JSON file located in app_data_dir.
        """
        try:
            with open(self.presets_file, 'w', encoding='utf-8') as file:
                json.dump(self.presets, file, ensure_ascii=False, indent=4)
        except Exception as e:
            messagebox.showerror("Error", f"프리셋을 저장하는 중 오류가 발생했습니다: {e}")

    def save_and_close(self):
        """
        Collect data from text fields, concatenate it, save, and close the popup.
        """
        # Define sheet_name, excel_file, user_token as needed
        # For demonstration, we'll use placeholder values

        # Collect the remaining data
        passage = self.text_edits[0].get("1.0", "end-1c").strip()    # 지문
        question = self.text_edits[1].get("1.0", "end-1c").strip()   # 문제
        answer = self.text_edits[2].get("1.0", "end-1c").strip()     # 정답
        explanation = self.text_edits[3].get("1.0", "end-1c").strip() # 해설

        print(f"explanation: {explanation}")
        if explanation == "":
            explanation = "(Write your own explanation for the answer(s))"

        # Validation: Ensure all fields are filled
        if not passage or not question or not answer:
            messagebox.showwarning(
                "Warning", 
                "'지문', '문제', '정답'란을 모두 채워주세요."  # "Please fill in all fields."
            )
            return

        # Concatenate the content as specified
        copy_question_content = (
            "[Passage]\n" + passage + 
            "\n[Question]\n" + question + 
            "\n[Answer]\n" + answer + 
            "\n[Explanation]\n" + explanation
        )

        # Save the concatenated content to the last_copy_question_file
        try:
            with open(self.last_copy_question_file, 'w', encoding='utf-8') as file:
                file.write(copy_question_content)
        except Exception as e:
            messagebox.showerror("Error", f"Failed to save copy question content: {e}")
            return

        # Optionally, you can also save the presets or perform other actions here

        # Close the popup
        self.destroy()

# Test Block to Run the Popup Independently
if __name__ == "__main__":
    import sys

    # Initialize CustomTkinter appearance and theme
    CTk.set_appearance_mode("System")  # Options: "System" (default), "Dark", "Light"
    CTk.set_default_color_theme("blue")  # Options: "blue" (default), "green", "dark-blue"

    # Define dummy functions for testing
    def load_window_size():
        """
        Dummy function to return window size.
        """
        return (1250, 680)  # Width x Height

    def load_font_size():
        """
        Dummy function to return font size.
        """
        return 12  # Default font size

    # Initialize the main Tkinter root window
    root = CTk.CTk()
    root.withdraw()  # Hide the root window

    # Instantiate the CopyQuestionPopup
    popup = CopyQuestionPopup(
        parent=root,
    )

    # Start the popup's event loop
    popup.mainloop()
