#modules/load_questions.py
import requests
from io import BytesIO
from openpyxl import load_workbook, Workbook
import os
import customtkinter
from tkinter import messagebox, filedialog
from tkinter.filedialog import asksaveasfilename
import unicodedata


class LoadQuestionsPopup(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, treeview=None, row_to_treeview_id_map=None, base_url=None):
        super().__init__(parent)
        self.title('모의고사 지문 불러오기')
        self.geometry('300x200')
        self.minsize(300, 200)

        self.parent = parent
        self.excel_file = excel_file
        self.treeview = treeview  # Reference to the Treeview
        self.row_to_treeview_id_map = row_to_treeview_id_map
        self.default_font = default_font
        self.base_url = base_url
        self.mocktest_file = None
        self.file_saved = None

        self.grade_map = {
            "1학년": "고1",
            "2학년": "고2",
            "3학년": "고3"
        }

        # Frame setup
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # ----- Row 1: 학년 -----
        grade_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        grade_frame.pack(side="top", fill="x", pady=5)
        grade_label = customtkinter.CTkLabel(
            grade_frame, 
            text="학년", 
            font=(self.default_font, 13, 'bold'), 
            text_color="black",
            width=100,
        )
        grade_label.pack(side="left", padx=(0,10))
        self.grade_optionmenu = customtkinter.CTkOptionMenu(
            grade_frame, 
            values=["1학년", "2학년", "3학년"],
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black",
            button_color="#DDA15C",
            button_hover_color="#BB6C25",
            command=self.on_grade_changed
        )
        self.grade_optionmenu.pack(side="right", fill="x", expand=True)

        # ----- Row 2: 연도(시행년도) -----
        year_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        year_frame.pack(side="top", fill="x", pady=5)
        year_label = customtkinter.CTkLabel(
            year_frame, 
            text="연도 (시행년도)", 
            font=(self.default_font, 13, 'bold'), 
            text_color="black",
            width=100,
        )
        year_label.pack(side="left", padx=(0,10))

        # Create the option menu with dummy values initially
        self.year_optionmenu = customtkinter.CTkOptionMenu(
            year_frame, 
            values=["Loading..."],  # Temporary placeholder
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black",
            button_color="#DDA15C",
            button_hover_color="#BB6C25",
            command=self.on_year_changed
        )
        self.year_optionmenu.pack(side="right", fill="x", expand=True)

        # ----- Row 3: 월 -----
        month_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        month_frame.pack(side="top", fill="x", pady=5)
        month_label = customtkinter.CTkLabel(
            month_frame, 
            text="월 (시행월)", 
            font=(self.default_font, 13, 'bold'), 
            text_color="black",
            width=100,
        )
        month_label.pack(side="left", padx=(0,10))
        self.month_optionmenu = customtkinter.CTkOptionMenu(
            month_frame, 
            values=["Loading..."],
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black",
            button_color="#DDA15C",
            button_hover_color="#BB6C25"
        )
        self.month_optionmenu.pack(side="right", fill="x", expand=True)

        # ----- Row 4: Button -----
        button_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        button_frame.pack(side="top", fill="x", pady=10)
        load_button = customtkinter.CTkButton(
            button_frame,
            text="대기열로 불러오기",
            font=(self.default_font, 13, 'bold'),
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_load_button_click
        )
        load_button.pack(side="top", pady=5)

        # After initializing UI, load the year options from the server
        self.load_year_options_from_server()

        self.bind("<Escape>", self.on_close)
        self.bind("<Return>", self.on_load_button_click)

    def load_year_options_from_server(self):
        # Fetch the list of available years from the Flask app
        if not self.base_url:
            print("[DEBUG] base_url not provided, cannot fetch year list.")
            return

        api_url = f"{self.base_url}/list_excel_files"
        response = requests.get(api_url)
        if response.status_code != 200:
            print("[DEBUG] Could not fetch year list from server")
            return

        data = response.json()
        years = data.get("years", [])

        if not years:
            years = ["데이터가 없음"]  # Fallback if no files found

        self.year_optionmenu.configure(values=years)

        # Select the first value by default if available
        if years:
            self.year_optionmenu.set(years[0])
            # Trigger the callback to populate months based on the current year and grade
            self.on_year_changed(years[0])

    def on_year_changed(self, new_year):
        # Whenever the year is changed, update the month options based on current grade
        current_year = self.year_optionmenu.get()
        current_grade_label = self.grade_optionmenu.get()
        self.grade = self.grade_map[current_grade_label]

        # Update month options dynamically
        self.update_month_options(current_year, self.grade)

    def on_grade_changed(self, new_grade):
        # Whenever the grade is changed, update the month options based on current year
        current_year = self.year_optionmenu.get()
        current_grade_label = self.grade_optionmenu.get()
        self.grade = self.grade_map[current_grade_label]

        # Update month options dynamically
        self.update_month_options(current_year, self.grade)


    def update_month_options(self, year_str, grade_str):

        # Extract last two digits for comparison
        year_two_digits = year_str[-2:]

        api_url = f"{self.base_url}/get_excel_file/{year_str}.xlsx"
        response = requests.get(api_url)
        if response.status_code != 200:
            self.month_optionmenu.configure(values=[""])
            return

        file_bytes = BytesIO(response.content)
        try:
            wb = load_workbook(file_bytes, read_only=True, data_only=True)
            sheet_name = "대기열"
            if sheet_name not in wb.sheetnames:
                self.month_optionmenu.configure(values=[""])
                return
            sheet = wb[sheet_name]
        except Exception as e:
            self.month_optionmenu.configure(values=[""])
            return

        matched_months = set()

        for row_idx, row in enumerate(sheet.iter_rows(values_only=True, min_row=1, max_col=1), start=1):
            cell_value = row[0]
            if cell_value and isinstance(cell_value, str):
                parts = cell_value.split('-')
                # We expect something like ["24", "03", "고1", ...]
                if len(parts) >= 3:
                    # Compare using year_two_digits and grade_str
                    if parts[0] == year_two_digits and parts[2] == grade_str:
                        mm = parts[1]
                        if mm.isdigit() and len(mm) == 2:
                            matched_months.add(mm)

        sorted_months = sorted(matched_months, key=lambda x: int(x))

        month_values = []
        for m in sorted_months:
            m_int = int(m)
            month_values.append(f"{m_int}월")

        if not month_values:
            print("[DEBUG] No matching months found. Setting month_optionmenu to empty.")
            month_values = [""]

        self.month_optionmenu.configure(values=month_values)
        if month_values:
            self.month_optionmenu.set(month_values[0])



    def on_load_button_click(self, event=None):
        # Fetch the selected values
        selected_year = self.year_optionmenu.get()  # e.g. "2024"
        selected_month = self.month_optionmenu.get() # e.g. "3월"
        selected_grade = self.grade_optionmenu.get() # e.g. "1학년"

        full_year_str = selected_year
        year_str = full_year_str[-2:]

        # Set mocktest_file using the full year
        self.mocktest_file = full_year_str + ".xlsx"

        # Convert the selected_month back to two-digit format if possible
        # e.g. "3월" -> "03"
        month_num_str = selected_month.replace("월", "").strip()
        if month_num_str.isdigit():
            month_int = int(month_num_str)
            month_str = f"{month_int:02d}"
        else:
            # If no proper month is selected, fallback or show error
            messagebox.showerror("오류", "유효한 월을 선택하세요.")
            return

        # Convert 학년 to 고1, 고2, 고3
        grade_str = self.grade_map[selected_grade]

        # Combine into 연도_월_학년 (prefix)
        prefix = f"{year_str}-{month_str}-{grade_str}"



        if not self.excel_file:
            messagebox.showinfo("Info", "현재 사용중인 파일이 없습니다. 문제를 불러오기 전에 파일을 먼저 저장해야 합니다.")                        

            self.new_새파일저장묻기()

            if self.file_saved == True:
                self.excel_file = unicodedata.normalize('NFC', self.excel_file)
                self.parent.app_logger.info(f"★[알림] 저장 성공\n(파일 경로: {self.excel_file})\n삭제/덮어쓰기 주의!\n★Make Questions를 클릭하여 출제를 시작하세요.\n")
                self.parent.filepath_label.configure(text=f"현재 파일 경로: {self.excel_file}")
                # Add the file to recent files list
                self.parent.add_to_recent_files(self.excel_file)
                self.parent.load_file_from_server(self.excel_file)
            else:
                messagebox.showerror("오류", "파일이 저장되지 않았습니다. 현재 파일을 먼저 저장해야 합니다.")
                return




        self.parent.sheet_selector.set("대기열")
        self.parent.load_excel_into_treeview(self.excel_file, "대기열")

        # Add matched data to the treeview (only columns A and B)
        self.add_matched_data_to_treeview(self.mocktest_file, "대기열", prefix)

        self.parent.save_treeview_to_excel(self.excel_file, "대기열")
        self.parent.app_logger.info("지문 불러오기를 완료했습니다.")

        self.destroy()

    def add_matched_data_to_treeview(self, excel_file, sheet_name, prefix):
        if not excel_file or not sheet_name:
            print("[DEBUG] Error: excel_file 혹은 sheet_name이 없음")
            return

        api_url = f"{self.base_url}/get_excel_file/{os.path.basename(excel_file)}"
        response = requests.get(api_url)
        if response.status_code != 200:
            print(f"[DEBUG] Error: Could not fetch the Excel file from {api_url}")
            messagebox.showerror("오류", "서버에 접근할 수 없습니다. 관리자에게 문의하세요.")
            return

        file_bytes = BytesIO(response.content)
        try:
            wb = load_workbook(file_bytes, read_only=True, data_only=True)
            if sheet_name not in wb.sheetnames:
                print(f"[DEBUG] Sheet {sheet_name} not found in the workbook.")
                messagebox.showerror("오류", "요청한 시트가 서버에 존재하지 않습니다. 관리자에게 문의하세요.")
                return
            sheet = wb[sheet_name]
        except Exception as e:
            print(f"[DEBUG] Error loading workbook: {e}")
            messagebox.showerror("오류", "엑셀 파일을 불러오는 중 오류가 발생했습니다. 관리자에게 문의하세요.")
            return

        # Find the last non-empty row in treeview column A or B
        all_items = self.treeview.get_children()
        last_nonempty_index = 0
        for idx, item_id in enumerate(all_items, start=1):
            row_values = self.treeview.item(item_id, "values")
            if (row_values[0].strip() != "") or (row_values[1].strip() != ""):
                last_nonempty_index = idx


        # Collect all existing values in column A to detect duplicates
        existing_values = set()
        for item_id in all_items:
            row_values = self.treeview.item(item_id, "values")
            if row_values and row_values[0].strip():
                existing_values.add(row_values[0].strip())

        insert_position = last_nonempty_index
        matched_rows = 0
        duplicates = []  # To store duplicates that won't be inserted

        # Fetch only columns A and B from the sheet
        for row_index, row in enumerate(sheet.iter_rows(values_only=True, min_row=1, max_col=2), start=1):
            formatted_row = ["" if cell is None else cell for cell in row]
            while len(formatted_row) < 5:
                formatted_row.append("")

            first_col = str(formatted_row[0])
            if first_col.startswith(prefix):
                # Check for duplicates
                if first_col.strip() in existing_values:
                    duplicates.append(first_col.strip())
                else:
                    item_id = self.treeview.insert("", insert_position, values=formatted_row[:5])
                    insert_position += 1
                    matched_rows += 1
                    existing_values.add(first_col.strip())


        # If we found duplicates, show one error message listing all duplicates
        if duplicates:
            duplicates_str = ", ".join(duplicates)
            messagebox.showerror("오류", f"{{{duplicates_str}}}는 이미 대기열에 있어 불러오지 않습니다.")

        # If no rows matched the prefix and no duplicates found
        if matched_rows == 0 and not duplicates:
            messagebox.showerror("오류", "서버에 데이터가 존재하지 않습니다. 관리자에게 문의하세요.")
            return

        # Retag rows (even/odd)
        self.retag_rows()

        current_row_count = len(self.treeview.get_children())
        empty_rows_to_add = max(0, 30 - current_row_count)

        for _ in range(empty_rows_to_add):
            self.treeview.insert("", "end", values=["", "", "", "", ""])
            current_row_count += 1

        self.retag_rows()

        self.row_to_treeview_id_map.clear()
        for new_index, item in enumerate(self.treeview.get_children(), start=1):
            self.row_to_treeview_id_map[new_index] = item

        self.update_scrollbars()

    def retag_rows(self):
        for index, item in enumerate(self.treeview.get_children()):
            if index % 2 == 0:
                self.treeview.item(item, tags=('evenrow',))
            else:
                self.treeview.item(item, tags=('oddrow',))

        self.treeview.tag_configure('evenrow', background='#F5F5F5')
        self.treeview.tag_configure('oddrow', background='#FAFAFA')

    def update_scrollbars(self):
        # Implement if you have scrollbars that need updating
        pass

    def on_close(self, event=None):
        self.destroy()

    def new_새파일저장묻기(self, event=None):
        
        if self.excel_file is None:
            file_path = filedialog.asksaveasfilename(defaultextension=".xlsx", filetypes=[("Excel files", "*.xlsx"), ("All files", "*.*")])                
            
            if not file_path:  # If the user cancels the save dialog
                self.parent.app_logger.info(f"저장이 취소되었습니다.\n")
                return  # Do not proceed further

            # Create a new Excel workbook and add a default sheet
            self.file_saved = True
            wb = Workbook()
            ws = wb.active
            ws.title = "대기열"
            # Save the new workbook to the chosen file path
            wb.save(filename=file_path)
            self.excel_file = file_path  # Update the global excel_file variable
            

            sheet_name = "대기열"  
            self.parent.save_treeview_to_excel(self.excel_file, sheet_name)
            self.parent.update_sheet_selector(self.excel_file)
            
